/*
 * Author: Andreas Ericsson <ae@op5.se>
 * Copyleft: GPLv2 (http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt)
 */

#include <nagios/broker.h>
#include <nagios/nebcallbacks.h>
#include <nagios/nebstructs.h>

NEB_API_VERSION(CURRENT_NEB_API_VERSION);

static FILE *cmd_pipe;

static FILE *open_command_pipe(const char *cmd)
{
	cmd_pipe = popen(cmd, "w");
	if (!cmd_pipe)
		fprintf(stderr, "cmd_pipe is NULL: %s\n", strerror(errno));

	return cmd_pipe;
}

/*
 * From the nsca README:
 * Service check information is read from standard
 * input in tab-delimited format as follows:
 *
 *   <host_name>[tab]<svc_description>[tab]<exit_code>[tab]<plugin_output>\n
 *
 * Host check information is submitted in a similiar
 * fashion - just leave out the service description:
 *
 *   <host_name>[tab]<return_code>[tab]<plugin_output>\n
 */

/* Our hooks. One for hosts, one for services */
static int host_hook(int cb, void *data)
{
	nebstruct_host_check_data *ds = (nebstruct_host_check_data *)data;

	/* ignore unprocessed checks */
	if (ds->type != NEBTYPE_HOSTCHECK_PROCESSED)
		return 0;

	fprintf(cmd_pipe, "%s\t%d\t%s\n", ds->host_name, ds->state, ds->output);

	return 0;
}

static int svc_hook(int cb, void *data)
{
	nebstruct_service_check_data *ds = (nebstruct_service_check_data *)data;

	/* ignore unprocessed checks */
	if (ds->type != NEBTYPE_SERVICECHECK_PROCESSED)
		return 0;

	fprintf(cmd_pipe, "%s\t%s\t%d\t%s\n",
	        ds->host_name, ds->service_description,
	        ds->state, ds->output);

	return 0;
}

/* init / deinit couple. These can't be static */
int nebmodule_init(int flags, char *arg, nebmodule *mod)
{
	open_command_pipe(arg);
	neb_register_callback(NEBCALLBACK_HOST_CHECK_DATA, mod, 0, host_hook);
	neb_register_callback(NEBCALLBACK_SERVICE_CHECK_DATA, mod, 0, svc_hook);
	return 0;
}

int nebmodule_deinit(int flags, int reason)
{
	neb_deregister_callback(NEBCALLBACK_HOST_CHECK_DATA, host_hook);
	neb_deregister_callback(NEBCALLBACK_SERVICE_CHECK_DATA, svc_hook);
	return 0;
}
